@@ -2,8 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2022 Osimis S.A., Belgium
- * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -85,7 +85,7 @@ namespace Orthanc
     return instanceUid_;
   }
 
-  const std::string& DicomInstanceHasher::HashPatient()
+  const std::string& DicomInstanceHasher::HashPatient() const
   {
     if (patientHash_.size() == 0)
     {
@@ -95,7 +95,7 @@ namespace Orthanc
     return patientHash_;
   }
 
-  const std::string& DicomInstanceHasher::HashStudy()
+  const std::string& DicomInstanceHasher::HashStudy() const
   {
     if (studyHash_.size() == 0)
     {
@@ -105,7 +105,7 @@ namespace Orthanc
     return studyHash_;
   }
 
-  const std::string& DicomInstanceHasher::HashSeries()
+  const std::string& DicomInstanceHasher::HashSeries() const
   {
     if (seriesHash_.size() == 0)
     {
@@ -115,7 +115,7 @@ namespace Orthanc
     return seriesHash_;
   }
 
-  const std::string& DicomInstanceHasher::HashInstance()
+  const std::string& DicomInstanceHasher::HashInstance() const
   {
     if (instanceHash_.size() == 0)
     {
