@@ -2,7 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -26,6 +27,12 @@
 #include "../OrthancException.h"
 
 #include <cassert>
+#include <sstream>
+#include <boost/iostreams/device/array.hpp>
+#include <boost/iostreams/stream.hpp>
+
+
+#include <iostream>
 
 namespace Orthanc
 {
@@ -323,6 +330,9 @@ namespace Orthanc
     }
     else
     {
+      assert(reader_.GetProcessedBytes() >= block.size());
+      const uint64_t tagOffset = reader_.GetProcessedBytes() - block.size();
+        
       ValueRepresentation vr = ValueRepresentation_Unknown;
         
       if (transferSyntax_ == DicomTransferSyntax_LittleEndianImplicit)
@@ -331,6 +341,7 @@ namespace Orthanc
         {
           danglingTag_ = tag;
           danglingVR_ = vr;
+          danglingOffset_ = tagOffset;
         }
 
         uint32_t length = ReadUnsignedInteger32(block.c_str() + 4, true /* little endian */);
@@ -372,6 +383,7 @@ namespace Orthanc
         {
           danglingTag_ = tag;
           danglingVR_ = vr;
+          danglingOffset_ = tagOffset;
         }
       }
     }
@@ -400,12 +412,19 @@ namespace Orthanc
   }    
 
     
-  void DicomStreamReader::HandleDatasetExplicitLength(const std::string& block)
+  void DicomStreamReader::HandleDatasetExplicitLength(IVisitor& visitor,
+                                                      const std::string& block)
   {
     assert(block.size() == 4);
 
     uint32_t length = ReadUnsignedInteger32(block.c_str(), IsLittleEndian());
     HandleDatasetExplicitLength(length);
+
+    std::string empty;
+    if (!visitor.VisitDatasetTag(danglingTag_, danglingVR_, empty, IsLittleEndian(), danglingOffset_))
+    {
+      state_ = State_Done;
+    }
   }
     
 
@@ -451,13 +470,13 @@ namespace Orthanc
       if (IsNormalizationNeeded(block, danglingVR_))
       {
         std::string s(block.begin(), block.end() - 1);
-        c = visitor.VisitDatasetTag(danglingTag_, danglingVR_, s, IsLittleEndian());
+        c = visitor.VisitDatasetTag(danglingTag_, danglingVR_, s, IsLittleEndian(), danglingOffset_);
       }
       else
       {
-        c = visitor.VisitDatasetTag(danglingTag_, danglingVR_, block, IsLittleEndian());
+        c = visitor.VisitDatasetTag(danglingTag_, danglingVR_, block, IsLittleEndian(), danglingOffset_);
       }
-        
+      
       if (!c)
       {
         state_ = State_Done;
@@ -476,6 +495,7 @@ namespace Orthanc
     transferSyntax_(DicomTransferSyntax_LittleEndianImplicit),  // Dummy
     danglingTag_(0x0000, 0x0000),  // Dummy
     danglingVR_(ValueRepresentation_Unknown),  // Dummy
+    danglingOffset_(0),  // Dummy
     sequenceDepth_(0)
   {
     reader_.Schedule(128 /* empty header */ +
@@ -510,7 +530,7 @@ namespace Orthanc
             break;
 
           case State_DatasetExplicitLength:
-            HandleDatasetExplicitLength(block);
+            HandleDatasetExplicitLength(visitor, block);
             break;
 
           case State_SequenceExplicitLength:
@@ -554,4 +574,165 @@ namespace Orthanc
   {
     return reader_.GetProcessedBytes();
   }
+
+
+  class DicomStreamReader::PixelDataVisitor : public DicomStreamReader::IVisitor
+  {
+  private:
+    bool                 hasPixelData_;
+    uint64_t             pixelDataOffset_;
+    ValueRepresentation  pixelDataVR_;
+    DicomTransferSyntax  transferSyntax_;
+    
+  public:
+    PixelDataVisitor() :
+      hasPixelData_(false),
+      pixelDataOffset_(0),
+      pixelDataVR_(ValueRepresentation_Unknown),
+      transferSyntax_(DicomTransferSyntax_LittleEndianImplicit) // Default DICOM transfer syntax
+    {
+    }
+    
+    virtual void VisitMetaHeaderTag(const DicomTag& tag,
+                                    const ValueRepresentation& vr,
+                                    const std::string& value) ORTHANC_OVERRIDE
+    {
+    }
+
+    virtual void VisitTransferSyntax(DicomTransferSyntax transferSyntax) ORTHANC_OVERRIDE
+    {
+      transferSyntax_ = transferSyntax;
+    }
+    
+    virtual bool VisitDatasetTag(const DicomTag& tag,
+                                 const ValueRepresentation& vr,
+                                 const std::string& value,
+                                 bool isLittleEndian,
+                                 uint64_t fileOffset) ORTHANC_OVERRIDE
+    {
+      if (tag == DICOM_TAG_PIXEL_DATA)
+      {
+        hasPixelData_ = true;
+        pixelDataOffset_ = fileOffset;
+
+        if (transferSyntax_ == DicomTransferSyntax_LittleEndianImplicit)
+        {
+          // Implicit Little Endian has always "OW" VR for pixel data
+          // https://dicom.nema.org/medical/dicom/current/output/chtml/part05/chapter_A.html
+          pixelDataVR_ = ValueRepresentation_OtherWord;
+        }
+        else if (transferSyntax_ == DicomTransferSyntax_LittleEndianExplicit ||
+                 transferSyntax_ == DicomTransferSyntax_BigEndianExplicit)
+        {
+          pixelDataVR_ = vr;
+        }
+        else
+        {
+          // Compressed transfer syntaxes must always be OB
+          pixelDataVR_ = ValueRepresentation_OtherByte;
+        }
+      }
+
+      // Stop processing once pixel data has been passed
+      return (tag < DICOM_TAG_PIXEL_DATA);
+    }
+
+    bool HasPixelData() const
+    {
+      return hasPixelData_;
+    }
+
+    uint64_t GetPixelDataOffset() const
+    {
+      return pixelDataOffset_;
+    }
+
+    ValueRepresentation GetPixelDataVR() const
+    {
+      return pixelDataVR_;
+    }
+
+    static bool LookupPixelDataOffset(uint64_t& offset,
+                                      ValueRepresentation& vr,
+                                      std::istream& stream)
+    {
+      PixelDataVisitor visitor;
+      bool isLittleEndian;
+
+      {
+        DicomStreamReader reader(stream);
+
+        try
+        {
+          reader.Consume(visitor);
+          isLittleEndian = reader.IsLittleEndian();
+        }
+        catch (OrthancException& e)
+        {
+          // Invalid DICOM file
+          return false;
+        }
+      }
+
+      if (visitor.HasPixelData())
+      {
+        // Sanity check if we face an unsupported DICOM file: Make
+        // sure that we can read DICOM_TAG_PIXEL_DATA at the reported
+        // position in the stream
+        stream.seekg(visitor.GetPixelDataOffset(), stream.beg);
+        
+        std::string s;
+        s.resize(4);
+        stream.read(&s[0], s.size());
+
+        if (!isLittleEndian)
+        {
+          // Byte swapping if reading a file whose transfer syntax is
+          // 1.2.840.10008.1.2.2 (big endian explicit)
+          std::swap(s[0], s[1]);
+          std::swap(s[2], s[3]);          
+        }
+        
+        if (stream.gcount() == static_cast<std::streamsize>(s.size()) &&
+            s[0] == char(0xe0) &&
+            s[1] == char(0x7f) &&
+            s[2] == char(0x10) &&
+            s[3] == char(0x00))
+        {
+          offset = visitor.GetPixelDataOffset();
+          vr = visitor.GetPixelDataVR();
+          return true;
+        }
+        else
+        {
+          return false;
+        }
+      }
+      else
+      {
+        return false;
+      }
+    }
+  };
+
+  
+  bool DicomStreamReader::LookupPixelDataOffset(uint64_t& offset,
+                                                ValueRepresentation& vr,
+                                                const std::string& dicom)
+  {
+    std::stringstream stream(dicom);
+    return PixelDataVisitor::LookupPixelDataOffset(offset, vr, stream);
+  }
+  
+
+  bool DicomStreamReader::LookupPixelDataOffset(uint64_t& offset,
+                                                ValueRepresentation& vr,
+                                                const void* buffer,
+                                                size_t size)
+  {
+    boost::iostreams::array_source source(reinterpret_cast<const char*>(buffer), size);
+    boost::iostreams::stream<boost::iostreams::array_source> stream(source);
+    return PixelDataVisitor::LookupPixelDataOffset(offset, vr, stream);
+  }
 }
+
