@@ -2,8 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2022 Osimis S.A., Belgium
- * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -40,7 +40,8 @@ namespace Orthanc
 
   DicomValue::DicomValue(const DicomValue& other) :
     type_(other.type_),
-    content_(other.content_)
+    content_(other.content_),
+    sequenceJson_(other.sequenceJson_)
   {
   }
 
@@ -61,10 +62,19 @@ namespace Orthanc
     content_.assign(data, size);
   }
     
+  DicomValue::DicomValue(const Json::Value& value) :
+    type_(Type_SequenceAsJson),
+    sequenceJson_(value)
+  {
+    if (value.type() != Json::arrayValue)
+    {
+      throw OrthancException(ErrorCode_BadParameterType);
+    }
+  }
   
   const std::string& DicomValue::GetContent() const
   {
-    if (type_ == Type_Null)
+    if (type_ == Type_Null || type_ == Type_SequenceAsJson)
     {
       throw OrthancException(ErrorCode_BadParameterType);
     }
@@ -74,6 +84,19 @@ namespace Orthanc
     }
   }
 
+  const Json::Value& DicomValue::GetSequenceContent() const
+  {
+    if (type_ != Type_SequenceAsJson)
+    {
+      throw OrthancException(ErrorCode_BadParameterType);
+    }
+    else
+    {
+      return sequenceJson_;
+    }
+  }
+
+
   bool DicomValue::IsNull() const
   {
     return type_ == Type_Null;
@@ -84,6 +107,15 @@ namespace Orthanc
     return type_ == Type_Binary;
   }
 
+  bool DicomValue::IsString() const
+  {
+    return type_ == Type_String;
+  }
+
+  bool DicomValue::IsSequence() const
+  {
+    return type_ == Type_SequenceAsJson;
+  }
 
   DicomValue* DicomValue::Clone() const
   {
@@ -107,8 +139,7 @@ namespace Orthanc
 
   bool DicomValue::ParseInteger32(int32_t& result) const
   {
-    if (IsBinary() ||
-        IsNull())
+    if (!IsString())
     {
       return false;
     }
@@ -120,8 +151,7 @@ namespace Orthanc
 
   bool DicomValue::ParseInteger64(int64_t& result) const
   {
-    if (IsBinary() ||
-        IsNull())
+    if (!IsString())
     {
       return false;
     }
@@ -133,8 +163,7 @@ namespace Orthanc
 
   bool DicomValue::ParseUnsignedInteger32(uint32_t& result) const
   {
-    if (IsBinary() ||
-        IsNull())
+    if (!IsString())
     {
       return false;
     }
@@ -146,8 +175,7 @@ namespace Orthanc
 
   bool DicomValue::ParseUnsignedInteger64(uint64_t& result) const
   {
-    if (IsBinary() ||
-        IsNull())
+    if (!IsString())
     {
       return false;
     }
@@ -159,8 +187,7 @@ namespace Orthanc
 
   bool DicomValue::ParseFloat(float& result) const
   {
-    if (IsBinary() ||
-        IsNull())
+    if (!IsString())
     {
       return false;
     }
@@ -172,8 +199,7 @@ namespace Orthanc
 
   bool DicomValue::ParseDouble(double& result) const
   {
-    if (IsBinary() ||
-        IsNull())
+    if (!IsString())
     {
       return false;
     }
@@ -185,8 +211,7 @@ namespace Orthanc
 
   bool DicomValue::ParseFirstFloat(float& result) const
   {
-    if (IsBinary() ||
-        IsNull())
+    if (!IsString())
     {
       return false;
     }
@@ -200,8 +225,7 @@ namespace Orthanc
   {
     uint64_t value;
 
-    if (IsBinary() ||
-        IsNull())
+    if (!IsString())
     {
       return false;
     }
@@ -223,6 +247,10 @@ namespace Orthanc
     {
       return false;
     }
+    else if (IsSequence())
+    {
+      return false;
+    }
     else if (IsBinary() && !allowBinary)
     {
       return false;
@@ -263,6 +291,11 @@ namespace Orthanc
         break;
       }
 
+      case Type_SequenceAsJson:
+      {
+        throw OrthancException(ErrorCode_NotImplemented);
+      }
+
       default:
         throw OrthancException(ErrorCode_InternalError);
     }
@@ -289,6 +322,10 @@ namespace Orthanc
       const std::string base64 =SerializationToolbox::ReadString(source, KEY_CONTENT);
       Toolbox::DecodeBase64(content_, base64);
     }
+    else if (type == "Sequence")
+    {
+      throw OrthancException(ErrorCode_NotImplemented);
+    }
     else
     {
       throw OrthancException(ErrorCode_BadFileFormat);
