@@ -2,8 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2022 Osimis S.A., Belgium
- * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -43,11 +43,13 @@ namespace Orthanc
     {
       Type_Null,
       Type_String,
-      Type_Binary
+      Type_Binary,
+      Type_SequenceAsJson
     };
 
     Type         type_;
     std::string  content_;
+    Json::Value  sequenceJson_;
 
     DicomValue(const DicomValue& other);
 
@@ -61,11 +63,19 @@ namespace Orthanc
                size_t size,
                bool isBinary);
     
+    explicit DicomValue(const Json::Value& value);
+    
     const std::string& GetContent() const;
 
+    const Json::Value& GetSequenceContent() const;
+
     bool IsNull() const;
 
     bool IsBinary() const;
+
+    bool IsString() const;
+
+    bool IsSequence() const;
     
     DicomValue* Clone() const;
 
