@@ -2,8 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2022 Osimis S.A., Belgium
- * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -128,6 +128,7 @@ namespace Orthanc
     
     SetOfTags removals_;
     SetOfTags clearings_;
+    SetOfTags keep_;
     Replacements replacements_;
     bool removePrivateTags_;
     ResourceType level_;
@@ -178,6 +179,8 @@ namespace Orthanc
 
     void SetupAnonymization2021b();
 
+    void SetupAnonymization2023b();
+
     void UnserializeUidMap(ResourceType level,
                            const Json::Value& serialized,
                            const char* field);
@@ -208,10 +211,14 @@ namespace Orthanc
 
     bool IsReplaced(const DicomTag& tag) const;
 
+    void GetReplacedTags(std::set<DicomTag>& target) const;
+
     const Json::Value& GetReplacement(const DicomTag& tag) const;
 
     std::string GetReplacementAsString(const DicomTag& tag) const;
 
+    bool IsKept(const DicomTag& tag) const;
+
     void SetRemovePrivateTags(bool removed);
 
     bool ArePrivateTagsRemoved() const;
