@@ -2,8 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2022 Osimis S.A., Belgium
- * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -97,6 +97,9 @@ namespace Orthanc
 
     bool EmbedContentInternal(const std::string& dataUriScheme);
 
+    void EncapsulateDocument(MimeType mime,
+                             const std::string& document);
+
     // For internal use only, in order to provide const-correctness on
     // the top of DCMTK API
     DcmFileFormat& GetDcmtkObjectConst() const;
@@ -205,6 +208,7 @@ namespace Orthanc
     void SaveToFile(const std::string& path);
 #endif
 
+    // This method must only be used on the PixelData and EncapsulatedDocument tags
     void EmbedContent(const std::string& dataUriScheme);
 
     void EmbedImage(const ImageAccessor& accessor);
@@ -236,8 +240,6 @@ namespace Orthanc
 
     bool HasTag(const DicomTag& tag) const;
 
-    void EmbedPdf(const std::string& pdf);
-
     bool ExtractPdf(std::string& pdf) const;
 
     void GetRawFrame(std::string& target, // OUT
@@ -309,5 +311,12 @@ namespace Orthanc
 
     ImageAccessor* DecodeAllOverlays(int& originX,
                                      int& originY) const;
+
+    void InjectEmptyPixelData(ValueRepresentation vr);
+
+    // Remove all the tags after pixel data
+    void RemoveFromPixelData();
+
+    ValueRepresentation GuessPixelDataValueRepresentation() const;
   };
 }
