@@ -38,65 +38,91 @@ namespace Orthanc
   {
     return uuid + ":" + boost::lexical_cast<std::string>(contentType) + ":1";
   }
-  
+
+
   static std::string GetCacheKeyStartRange(const std::string& uuid,
                                            FileContentType contentType)
   {
     return uuid + ":" + boost::lexical_cast<std::string>(contentType) + ":0";
   }
-  
+
+
+  static std::string GetCacheKeyTranscodedInstance(const std::string& uuid,
+                                                   DicomTransferSyntax transferSyntax)
+  {
+    return uuid + ":ts:" + GetTransferSyntaxUid(transferSyntax);
+  }
+
+
   void StorageCache::SetMaximumSize(size_t size)
   {
     cache_.SetMaximumSize(size);
   }
   
 
-  void StorageCache::Add(const std::string& uuid, 
-                         FileContentType contentType,
-                         const std::string& value)
-  {
-    const std::string key = GetCacheKeyFullFile(uuid, contentType);
-    cache_.Add(key, value);
-  }
-  
-
-  void StorageCache::Add(const std::string& uuid, 
-                         FileContentType contentType,
-                         const void* buffer,
-                         size_t size)
-  {
-    const std::string key = GetCacheKeyFullFile(uuid, contentType);
-    cache_.Add(key, buffer, size);
-  }
-
-
-  void StorageCache::AddStartRange(const std::string& uuid, 
-                                   FileContentType contentType,
-                                   const std::string& value)
-  {
-    const std::string key = GetCacheKeyStartRange(uuid, contentType);
-    cache_.Add(key, value);
-  }
-
-
   void StorageCache::Invalidate(const std::string& uuid,
                                 FileContentType contentType)
   {
-    // invalidate both full file + start range file
+    std::set<DicomTransferSyntax> transferSyntaxes;
+
+    {
+      boost::mutex::scoped_lock lock(subKeysMutex_);
+      transferSyntaxes = subKeysTransferSyntax_;
+    }
+
+    // invalidate full file, start range file and possible transcoded instances
     const std::string keyFullFile = GetCacheKeyFullFile(uuid, contentType);
     cache_.Invalidate(keyFullFile);
 
     const std::string keyPartialFile = GetCacheKeyStartRange(uuid, contentType);
     cache_.Invalidate(keyPartialFile);
+    
+    for (std::set<DicomTransferSyntax>::const_iterator it = transferSyntaxes.begin(); it != transferSyntaxes.end(); ++it)
+    {
+      const std::string keyTransferSyntax = GetCacheKeyTranscodedInstance(uuid, *it);
+      cache_.Invalidate(keyTransferSyntax);
+    }
   }
-  
 
-  bool StorageCache::Fetch(std::string& value, 
-                           const std::string& uuid,
-                           FileContentType contentType)
+
+  StorageCache::Accessor::Accessor(StorageCache& cache)
+  : MemoryStringCache::Accessor(cache.cache_),
+    storageCache_(cache)
+  {
+  }
+
+  void StorageCache::Accessor::Add(const std::string& uuid, 
+                                   FileContentType contentType,
+                                   const std::string& value)
+  {
+
+    std::string key = GetCacheKeyFullFile(uuid, contentType);
+    MemoryStringCache::Accessor::Add(key, value);
+  }
+
+  void StorageCache::Accessor::AddStartRange(const std::string& uuid, 
+                                             FileContentType contentType,
+                                             const std::string& value)
+  {
+    const std::string key = GetCacheKeyStartRange(uuid, contentType);
+    MemoryStringCache::Accessor::Add(key, value);
+  }
+
+  void StorageCache::Accessor::Add(const std::string& uuid, 
+                                   FileContentType contentType,
+                                   const void* buffer,
+                                   size_t size)
   {
     const std::string key = GetCacheKeyFullFile(uuid, contentType);
-    if (cache_.Fetch(value, key))
+    MemoryStringCache::Accessor::Add(key, reinterpret_cast<const char*>(buffer), size);
+  }                                   
+
+  bool StorageCache::Accessor::Fetch(std::string& value, 
+                                     const std::string& uuid,
+                                     FileContentType contentType)
+  {
+    const std::string key = GetCacheKeyFullFile(uuid, contentType);
+    if (MemoryStringCache::Accessor::Fetch(value, key))
     {
       LOG(INFO) << "Read attachment \"" << uuid << "\" with content type "
                 << boost::lexical_cast<std::string>(contentType) << " from cache";
@@ -108,14 +134,44 @@ namespace Orthanc
     }
   }
 
-  bool StorageCache::FetchStartRange(std::string& value, 
-                                     const std::string& uuid,
-                                     FileContentType contentType,
-                                     uint64_t end)
+  bool StorageCache::Accessor::FetchTranscodedInstance(std::string& value, 
+                                                       const std::string& uuid,
+                                                       DicomTransferSyntax targetSyntax)
+  {
+    const std::string key = GetCacheKeyTranscodedInstance(uuid, targetSyntax);
+    if (MemoryStringCache::Accessor::Fetch(value, key))
+    {
+      LOG(INFO) << "Read instance \"" << uuid << "\" transcoded to "
+                << GetTransferSyntaxUid(targetSyntax) << " from cache";
+      return true;
+    }
+    else
+    {
+      return false;
+    }
+  }
+
+  void StorageCache::Accessor::AddTranscodedInstance(const std::string& uuid,
+                                                     DicomTransferSyntax targetSyntax,
+                                                     const void* buffer,
+                                                     size_t size)
+  {
+    {
+      boost::mutex::scoped_lock lock(storageCache_.subKeysMutex_);
+      storageCache_.subKeysTransferSyntax_.insert(targetSyntax);
+    }
+
+    const std::string key = GetCacheKeyTranscodedInstance(uuid, targetSyntax);
+    MemoryStringCache::Accessor::Add(key, reinterpret_cast<const char*>(buffer), size);
+  }
+
+  bool StorageCache::Accessor::FetchStartRange(std::string& value, 
+                                               const std::string& uuid,
+                                               FileContentType contentType,
+                                               uint64_t end /* exclusive */)
   {
-    // first try to get the start of file only from cache
     const std::string keyPartialFile = GetCacheKeyStartRange(uuid, contentType);
-    if (cache_.Fetch(value, keyPartialFile) && value.size() >= end)
+    if (MemoryStringCache::Accessor::Fetch(value, keyPartialFile) && value.size() >= end)
     {
       if (value.size() > end)  // the start range that has been cached is larger than the requested value
       {
@@ -126,23 +182,19 @@ namespace Orthanc
                 << boost::lexical_cast<std::string>(contentType) << " from cache";
       return true;
     }
-    else
-    {
-      // try to get the full file from cache
-      if (Fetch(value, uuid, contentType))
-      {
-        if (value.size() < end)
-        {
-          throw OrthancException(ErrorCode_CorruptedFile);
-        }
 
-        value.resize(end);
-        return true;
-      }
-      else
-      {
-        return false;
-      }
-    }
+    return false;
   }
+
+
+  size_t StorageCache::GetCurrentSize() const
+  {
+    return cache_.GetCurrentSize();
+  }
+  
+  size_t StorageCache::GetNumberOfItems() const
+  {
+    return cache_.GetNumberOfItems();
+  }
+
 }
