@@ -2,8 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2022 Osimis S.A., Belgium
- * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -106,6 +106,7 @@ namespace Orthanc
     uint16_t port_;
     IIncomingHttpRequestFilter* filter_;
     bool keepAlive_;
+    unsigned int keepAliveTimeout_;
     bool httpCompression_;
     IHttpExceptionFormatter* exceptionFormatter_;
     std::string realm_;
@@ -157,8 +158,12 @@ namespace Orthanc
 
     bool IsKeepAliveEnabled() const;
 
+    unsigned int GetKeepAliveTimeout() const;
+
     void SetKeepAliveEnabled(bool enabled);
 
+    void SetKeepAliveTimeout(unsigned int timeout);
+
     const std::string& GetSslCertificate() const;
 
     void SetSslCertificate(const char* path);
