@@ -38,7 +38,7 @@ namespace Orthanc
     void Stop();
 
   public:
-    explicit RunnableWorkersPool(size_t countWorkers);
+    explicit RunnableWorkersPool(size_t countWorkers, const std::string& name);
 
     ~RunnableWorkersPool();
 
