@@ -2,24 +2,13 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation, either version 3 of the
  * License, or (at your option) any later version.
- *
- * In addition, as a special exception, the copyright holders of this
- * program give permission to link the code of its release with the
- * OpenSSL project's "OpenSSL" library (or with modified versions of it
- * that use the same license as the "OpenSSL" library), and distribute
- * the linked executables. You must obey the GNU General Public License
- * in all respects for all of the code used other than "OpenSSL". If you
- * modify file(s) with this exception, you may extend this exception to
- * your version of the file(s), but you are not obligated to do so. If
- * you do not wish to do so, delete this exception statement from your
- * version. If you delete this exception statement from all source files
- * in the program, then also delete it here.
  * 
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -51,6 +40,36 @@ var LIMIT_RESOURCES = 100;
 var currentPage = '';
 var currentUuid = '';
 
+var ACQUISITION_NUMBER = '0020,0012';
+var IMAGES_IN_ACQUISITION = '0020,1002';
+var IMAGE_ORIENTATION_PATIENT = '0020,0037';
+var IMAGE_POSITION_PATIENT = '0020,0032';
+var INSTANCE_CREATION_DATE = '0008,0012';
+var INSTANCE_CREATION_TIME = '0008,0013';
+var INSTANCE_NUMBER = '0020,0013';
+var MANUFACTURER = '0008,0070';
+var OTHER_PATIENT_IDS = '0010,1000';
+var PATIENT_BIRTH_DATE = '0010,0030';
+var PATIENT_NAME = '0010,0010';
+var SERIES_DATE = '0008,0021';
+var SERIES_DESCRIPTION = '0008,103e';
+var SERIES_INSTANCE_UID = '0020,000e';
+var SERIES_TIME = '0008,0031';
+var SOP_INSTANCE_UID = '0008,0018';
+var STUDY_DATE = '0008,0020';
+var STUDY_DESCRIPTION = '0008,1030';
+var STUDY_INSTANCE_UID = '0020,000d';
+var STUDY_TIME = '0008,0030';
+
+var ANONYMIZED_FROM = 'AnonymizedFrom';
+var MODIFIED_FROM = 'ModifiedFrom';
+
+
+function IsAlphanumeric(s)
+{
+  return s.match(/^[0-9a-zA-Z]+$/);
+}
+
 
 function DeepCopy(obj)
 {
@@ -158,12 +177,6 @@ $.ajaxSetup(
 );
 
 
-function SplitLongUid(s)
-{
-  return '<span>' + s.substr(0, s.length / 2) + '</span> <span>' + s.substr(s.length / 2, s.length - s.length / 2) + '</span>';
-}
-
-
 function ParseDicomDate(s)
 {
   y = parseInt(s.substr(0, 4), 10);
@@ -259,10 +272,20 @@ function Sort(arr, fieldExtractor, isInteger, reverse)
 }
 
 
+function GetMainDicomTag(mainDicomTags, tag)
+{
+  if (tag in mainDicomTags) {
+    return mainDicomTags[tag].Value;
+  } else {
+    return '';
+  }
+}
+
+
 function SortOnDicomTag(arr, tag, isInteger, reverse)
 {
   return Sort(arr, function(a) { 
-    return a.MainDicomTags[tag];
+    return GetMainDicomTag(a.MainDicomTags, tag);
   }, isInteger, reverse);
 }
 
@@ -291,7 +314,8 @@ function CompleteFormatting(node, link, isReverse, count)
                     .text(count));
   }
   
-  if (link != null)
+  if (link != null &&
+      link)
   {
     node = $('<a>').attr('href', link).append(node);
 
@@ -316,27 +340,30 @@ function FormatMainDicomTags(target, tags, tagsToIgnore)
   {
     if (tagsToIgnore.indexOf(i) == -1)
     {
-      v = tags[i];
+      v = GetMainDicomTag(tags, i);
 
-      if (i == "PatientBirthDate" ||
-          i == "StudyDate" ||
-          i == "SeriesDate")
+      if (i == PATIENT_BIRTH_DATE ||
+          i == STUDY_DATE ||
+          i == SERIES_DATE ||
+          i == INSTANCE_CREATION_DATE)
       {
         v = FormatDicomDate(v);
       }
-      else if (i == "DicomStudyInstanceUID" ||
-               i == "DicomSeriesInstanceUID")
+      else if (i == STUDY_INSTANCE_UID ||
+               i == SERIES_INSTANCE_UID ||
+               i == SOP_INSTANCE_UID)
       {
-        v = SplitLongUid(v);
+        // Possibly split a long UID
+        // v = '<span>' + s.substr(0, s.length / 2) + '</span><span>' + s.substr(s.length / 2, s.length - s.length / 2) + '</span>';
       }
-      else if (i == "ImagePositionPatient" ||
-               i == "ImageOrientationPatient")
+      else if (i == IMAGE_POSITION_PATIENT ||
+               i == IMAGE_ORIENTATION_PATIENT)
       {
         v = FormatFloatSequence(v);
       }
       
       target.append($('<p>')
-                    .text(i + ': ')
+                    .text(tags[i].Name + ': ')
                     .append($('<strong>').text(v)));
     }
   }
@@ -345,11 +372,11 @@ function FormatMainDicomTags(target, tags, tagsToIgnore)
 
 function FormatPatient(patient, link, isReverse)
 {
-  var node = $('<div>').append($('<h3>').text(patient.MainDicomTags.PatientName));
+  var node = $('<div>').append($('<h3>').text(GetMainDicomTag(patient.MainDicomTags, PATIENT_NAME)));
 
   FormatMainDicomTags(node, patient.MainDicomTags, [ 
-    "PatientName"
-    // "OtherPatientIDs"
+    PATIENT_NAME
+    //,  OTHER_PATIENT_IDS
   ]);
     
   return CompleteFormatting(node, link, isReverse, patient.Studies.length);
@@ -365,20 +392,20 @@ function FormatStudy(study, link, isReverse, includePatient)
   if (includePatient) {
     label = study.Label;
   } else {
-    label = study.MainDicomTags.StudyDescription;
+    label = GetMainDicomTag(study.MainDicomTags, STUDY_DESCRIPTION);
   }
 
   node = $('<div>').append($('<h3>').text(label));
 
   if (includePatient) {
     FormatMainDicomTags(node, study.PatientMainDicomTags, [ 
-      'PatientName'
+      PATIENT_NAME
     ]);
   }
     
   FormatMainDicomTags(node, study.MainDicomTags, [ 
-     'StudyDescription', 
-     'StudyTime' 
+    STUDY_DESCRIPTION, 
+    STUDY_TIME
   ]);
 
   return CompleteFormatting(node, link, isReverse, study.Series.length);
@@ -402,18 +429,18 @@ function FormatSeries(series, link, isReverse)
   }
   
   node = $('<div>')
-      .append($('<h3>').text(series.MainDicomTags.SeriesDescription))
-      .append($('<p>').append($('<em>')
-                           .text('Status: ')
-                           .append($('<strong>').text(series.Status))));
-
+    .append($('<h3>').text(GetMainDicomTag(series.MainDicomTags, SERIES_DESCRIPTION)))
+    .append($('<p>').append($('<em>')
+                            .text('Status: ')
+                            .append($('<strong>').text(series.Status))));
+  
   FormatMainDicomTags(node, series.MainDicomTags, [ 
-     "SeriesDescription", 
-     "SeriesTime", 
-     "Manufacturer",
-     "ImagesInAcquisition",
-     "SeriesDate",
-     "ImageOrientationPatient"
+    SERIES_DESCRIPTION,
+    SERIES_TIME,
+    MANUFACTURER,
+    IMAGES_IN_ACQUISITION,
+    SERIES_DATE,
+    IMAGE_ORIENTATION_PATIENT
   ]);
     
   return CompleteFormatting(node, link, isReverse, c);
@@ -425,10 +452,10 @@ function FormatInstance(instance, link, isReverse)
   var node = $('<div>').append($('<h3>').text('Instance: ' + instance.IndexInSeries));
 
   FormatMainDicomTags(node, instance.MainDicomTags, [
-    "AcquisitionNumber", 
-    "InstanceNumber", 
-    "InstanceCreationDate", 
-    "InstanceCreationTime"
+    ACQUISITION_NUMBER,
+    INSTANCE_NUMBER,
+    INSTANCE_CREATION_DATE,
+    INSTANCE_CREATION_TIME,
   ]);
     
   return CompleteFormatting(node, link, isReverse);
@@ -443,7 +470,8 @@ $('[data-role="page"]').live('pagebeforeshow', function() {
     cache: false,
     success: function(s) {
       if (s.Name != "") {
-        $('.orthanc-name').html($('<a>')
+        $('.orthanc-name').empty();
+        $('.orthanc-name').append($('<a>')
                                 .addClass('ui-link')
                                 .attr('href', 'explorer.html')
                                 .text(s.Name)
@@ -457,6 +485,14 @@ $('[data-role="page"]').live('pagebeforeshow', function() {
       } else {
         $('.warning-insecure').hide();
       }
+
+      // New in Orthanc 1.12.0
+      if ('HasLabels' in s &&
+          s.HasLabels) {
+        $('#lookup-study-labels-div').show();
+      } else {
+        $('#lookup-study-labels-div').hide();
+      }
     }
   });
 });
@@ -474,24 +510,41 @@ $('#lookup').live('pagebeforeshow', function() {
   target.append($('<option>').attr('value', GenerateDicomDate(-31) + '-').text('Last 31 days'));
   target.append($('<option>').attr('value', GenerateDicomDate(-31 * 3) + '-').text('Last 3 months'));
   target.append($('<option>').attr('value', GenerateDicomDate(-365) + '-').text('Last year'));
+  target.append($('<option>').attr('value', 'specific').text('Specific date'));
   target.selectmenu('refresh');
 
   $('#lookup-result').hide();
+  $('#lookup-study-date-specific').hide();
+});
+
+
+$('#lookup-study-date').live('change', function() {
+  if ($(this).val() == 'specific') {
+    $('#lookup-study-date-specific').show();
+  } else {
+    $('#lookup-study-date-specific').hide();
+  }
 });
 
 
 $('#lookup-submit').live('click', function() {
-  var lookup;
+  var lookup, studyDate;
 
   $('#lookup-result').hide();
 
+  studyDate = $('#lookup-study-date').val();
+  if (studyDate == 'specific') {
+    studyDate = IsoToDicomDate($('#lookup-study-date-specific').val());
+  }
+  
   lookup = {
     'Level' : 'Study',
     'Expand' : true,
     'Limit' : LIMIT_RESOURCES + 1,
     'Query' : {
-      'StudyDate' : $('#lookup-study-date').val()
-    }
+      'StudyDate' : studyDate
+    },
+    'Full' : true
   };
 
   $('#lookup-form input').each(function(index, input) {
@@ -508,6 +561,13 @@ $('#lookup-submit').live('click', function() {
       else if (input.id == 'lookup-study-description') {
         lookup['Query']['StudyDescription'] = input.value;
       }
+      else if (input.id == 'lookup-study-date-specific') {
+        // Ignore
+      }
+      else if (input.id == 'lookup-study-labels') {
+        // New in Orthanc 1.12.0
+        lookup['Labels'] = input.value.split(' ');
+      }
       else {
         console.error('Unknown lookup field: ' + input.id);
       }
@@ -534,14 +594,14 @@ $('#lookup-submit').live('click', function() {
 
 
 $('#find-patients').live('pagebeforeshow', function() {
-  GetResource('/patients?expand&since=0&limit=' + (LIMIT_RESOURCES + 1), function(patients) {
+  GetResource('/patients?expand&since=0&limit=' + (LIMIT_RESOURCES + 1) + '&full', function(patients) {
     var target = $('#all-patients');
     var count, showAlert, p;
 
 
     $('li', target).remove();
     
-    SortOnDicomTag(patients, 'PatientName', false, false);
+    SortOnDicomTag(patients, PATIENT_NAME, false, false);
 
     if (patients.length <= LIMIT_RESOURCES) {
       count = patients.length;
@@ -579,8 +639,8 @@ function FormatListOfStudies(targetId, alertId, countId, studies)
   $('li', target).remove();
 
   for (var i = 0; i < studies.length; i++) {
-    patient = studies[i].PatientMainDicomTags.PatientName;
-    study = studies[i].MainDicomTags.StudyDescription;
+    patient = GetMainDicomTag(studies[i].PatientMainDicomTags, PATIENT_NAME);
+    study = GetMainDicomTag(studies[i].MainDicomTags, STUDY_DESCRIPTION);
 
     s = "";
     if (typeof patient === 'string') {
@@ -626,7 +686,7 @@ function FormatListOfStudies(targetId, alertId, countId, studies)
 
 
 $('#find-studies').live('pagebeforeshow', function() {
-  GetResource('/studies?expand&since=0&limit=' + (LIMIT_RESOURCES + 1), function(studies) {
+  GetResource('/studies?expand&since=0&limit=' + (LIMIT_RESOURCES + 1) + '&full', function(studies) {
     FormatListOfStudies('#all-studies', '#alert-studies', '#count-studies', studies);
   });
 });
@@ -648,6 +708,103 @@ function SetupAnonymizedOrModifiedFrom(buttonSelector, resource, resourceType, f
   }
 }
 
+function SetupAttachments(accessSelector, liClass, resourceId, resourceType) {
+  GetResource('/' + resourceType + '/' + resourceId + '/attachments?full', function(attachments) {
+    target = $(accessSelector);
+    $('.' + liClass).remove();
+    for (var key in attachments) {
+      if (attachments[key] >= 1024) {
+        target.append('<li data-icon="gear" class="' + liClass + '"><a href="#" id="' + attachments[key] + '">Download ' + key + '</a></li>')
+      }
+    }
+    target.listview('refresh');
+  });
+}
+
+
+
+function RefreshLabels(nodeLabels, resourceLevel, resourceId)
+{
+  GetResource('/' + resourceLevel + '/' + resourceId + '/labels', function(labels) {
+    nodeLabels.empty();
+    
+    if (labels.length > 0) {
+      nodeLabels.css('display', 'block');
+
+      for (var i = 0; i < labels.length; i++) {
+        var removeButton = $('<button>').text('X').attr('title', 'Remove label "' + labels[i] + '"');
+
+        removeButton.click({
+          label : labels[i],
+          nodeLabels : nodeLabels
+        }, function(s) {
+          $.ajax({
+            url: '../' + resourceLevel + '/' + resourceId + '/labels/' + s.data.label,
+            dataType: 'json',
+            type: 'DELETE',
+            success: function(ss) {
+              RefreshLabels(nodeLabels, resourceLevel, resourceId);
+            }
+          });
+        });
+        
+        nodeLabels.append($('<span>').text(labels[i] + ' ').addClass('label').append(removeButton));
+      }
+    } else {
+      nodeLabels.css('display', 'none');
+    }
+  });
+}
+
+
+function ConfigureLabels(target, system, resourceLevel, resourceId)
+{
+  if (system.HasLabels === true) {
+    var nodeLabels = $('<li>').append($('<div>'));
+    var addLabelButton = $('<a>').text('Add label');
+
+    RefreshLabels(nodeLabels, resourceLevel, resourceId);
+
+    addLabelButton.click(function(s) {
+      $('#dialog').simpledialog2({
+        mode: 'button',
+        animate: false,
+        headerText: 'Add label',
+        headerClose: true,
+        buttonPrommpt: 'Enter the new label',
+        buttonInput: true,
+        width: '100%',
+        buttons : {
+          'OK': {
+            click: function () {
+              var label = $.mobile.sdLastInput;
+              if (label.length > 0) {
+                if (IsAlphanumeric(label)) {
+                  $.ajax({
+                    url: '../' + resourceLevel + '/' + resourceId + '/labels/' + label,
+                    dataType: 'json',
+                    type: 'PUT',
+                    data: '',
+                    success: function(ss) {
+                      RefreshLabels(nodeLabels, resourceLevel, resourceId);
+                    }
+                  });
+                } else {
+                  alert('Error: Labels can only contain alphanumeric characters');
+                }
+              }
+            }
+          },
+        }
+      });
+    });
+
+    target
+      .append('<li data-role="list-divider">Labels</li>')
+      .append(nodeLabels)
+      .append($('<li>').attr('data-icon', 'plus').append(addLabelButton));
+  }
+}
 
 
 function RefreshPatient()
@@ -657,50 +814,57 @@ function RefreshPatient()
   if ($.mobile.pageData) {
     pageData = DeepCopy($.mobile.pageData);
 
-    GetResource('/patients/' + pageData.uuid, function(patient) {
-      GetResource('/patients/' + pageData.uuid + '/studies', function(studies) {
-        SortOnDicomTag(studies, 'StudyDate', false, true);
+    GetResource('/system', function(system) {
+      GetResource('/patients/' + pageData.uuid + '?full', function(patient) {
+        GetResource('/patients/' + pageData.uuid + '/studies?full', function(studies) {
+          SortOnDicomTag(studies, STUDY_DATE, false, true);
 
-        $('#patient-info li').remove();
-        $('#patient-info')
-          .append('<li data-role="list-divider">Patient</li>')
-          .append(FormatPatient(patient))
-          .listview('refresh');
+          $('#patient-info li').remove();
 
-        target = $('#list-studies');
-        $('li', target).remove();
-        
-        for (var i = 0; i < studies.length; i++) {
-          if (i == 0 || studies[i].MainDicomTags.StudyDate != studies[i - 1].MainDicomTags.StudyDate)
-          {
-            target.append($('<li>')
-                          .attr('data-role', 'list-divider')
-                          .text(FormatDicomDate(studies[i].MainDicomTags.StudyDate)));
+          var info = $('#patient-info')
+              .append('<li data-role="list-divider">Patient</li>')
+              .append(FormatPatient(patient));
+          ConfigureLabels(info, system, 'patients', patient.ID);
+          info.listview('refresh');
+
+          target = $('#list-studies');
+          $('li', target).remove();
+          
+          for (var i = 0; i < studies.length; i++) {
+            if (i == 0 ||
+                GetMainDicomTag(studies[i].MainDicomTags, STUDY_DATE) !=
+                GetMainDicomTag(studies[i - 1].MainDicomTags, STUDY_DATE))
+            {
+              target.append($('<li>')
+                            .attr('data-role', 'list-divider')
+                            .text(FormatDicomDate(GetMainDicomTag(studies[i].MainDicomTags, STUDY_DATE))));
+            }
+
+            target.append(FormatStudy(studies[i], '#study?uuid=' + studies[i].ID));
           }
 
-          target.append(FormatStudy(studies[i], '#study?uuid=' + studies[i].ID));
-        }
+          SetupAnonymizedOrModifiedFrom('#patient-anonymized-from', patient, 'patient', ANONYMIZED_FROM);
+          SetupAnonymizedOrModifiedFrom('#patient-modified-from', patient, 'patient', MODIFIED_FROM);
+          SetupAttachments('#patient-access', 'patient-attachment', pageData.uuid, 'patients');
 
-        SetupAnonymizedOrModifiedFrom('#patient-anonymized-from', patient, 'patient', 'AnonymizedFrom');
-        SetupAnonymizedOrModifiedFrom('#patient-modified-from', patient, 'patient', 'ModifiedFrom');
+          target.listview('refresh');
 
-        target.listview('refresh');
+          // Check whether this patient is protected
+          $.ajax({
+            url: '../patients/' + pageData.uuid + '/protected',
+            type: 'GET',
+            dataType: 'text',
+            async: false,
+            cache: false,
+            success: function (s) {
+              v = (s == '1') ? 'on' : 'off';
+              $('#protection').val(v).slider('refresh');
+            }
+          });
 
-        // Check whether this patient is protected
-        $.ajax({
-          url: '../patients/' + pageData.uuid + '/protected',
-          type: 'GET',
-          dataType: 'text',
-          async: false,
-          cache: false,
-          success: function (s) {
-            v = (s == '1') ? 'on' : 'off';
-            $('#protection').val(v).slider('refresh');
-          }
+          currentPage = 'patient';
+          currentUuid = pageData.uuid;
         });
-
-        currentPage = 'patient';
-        currentUuid = pageData.uuid;
       });
     });
   }
@@ -714,39 +878,47 @@ function RefreshStudy()
   if ($.mobile.pageData) {
     pageData = DeepCopy($.mobile.pageData);
 
-    GetResource('/studies/' + pageData.uuid, function(study) {
-      GetResource('/patients/' + study.ParentPatient, function(patient) {
-        GetResource('/studies/' + pageData.uuid + '/series', function(series) {
-          SortOnDicomTag(series, 'SeriesDate', false, true);
-
-          $('#study .patient-link').attr('href', '#patient?uuid=' + patient.ID);
-          $('#study-info li').remove();
-          $('#study-info')
-            .append('<li data-role="list-divider">Patient</li>')
-            .append(FormatPatient(patient, '#patient?uuid=' + patient.ID, true))
-            .append('<li data-role="list-divider">Study</li>')
-            .append(FormatStudy(study))
-            .listview('refresh');
-
-          SetupAnonymizedOrModifiedFrom('#study-anonymized-from', study, 'study', 'AnonymizedFrom');
-          SetupAnonymizedOrModifiedFrom('#study-modified-from', study, 'study', 'ModifiedFrom');
-
-          target = $('#list-series');
-          $('li', target).remove();
-          for (var i = 0; i < series.length; i++) {
-            if (i == 0 || series[i].MainDicomTags.SeriesDate != series[i - 1].MainDicomTags.SeriesDate)
-            {
-              target.append($('<li>')
-                            .attr('data-role', 'list-divider')
-                            .text(FormatDicomDate(series[i].MainDicomTags.SeriesDate)));
-            }
+    GetResource('/system', function(system) {
+      GetResource('/studies/' + pageData.uuid + '?full', function(study) {
+        GetResource('/patients/' + study.ParentPatient + '?full', function(patient) {
+          GetResource('/studies/' + pageData.uuid + '/series?full', function(series) {
+            SortOnDicomTag(series, SERIES_DATE, false, true);
             
-            target.append(FormatSeries(series[i], '#series?uuid=' + series[i].ID));
-          }
-          target.listview('refresh');
+            $('#study .patient-link').attr('href', '#patient?uuid=' + patient.ID);
+            $('#study-info li').remove();
 
-          currentPage = 'study';
-          currentUuid = pageData.uuid;
+            var info = $('#study-info')
+                .append('<li data-role="list-divider">Patient</li>')
+                .append(FormatPatient(patient, '#patient?uuid=' + patient.ID, true))
+                .append('<li data-role="list-divider">Study</li>')
+                .append(FormatStudy(study));
+            ConfigureLabels(info, system, 'studies', study.ID);
+            info.listview('refresh');
+
+            SetupAnonymizedOrModifiedFrom('#study-anonymized-from', study, 'study', ANONYMIZED_FROM);
+            SetupAnonymizedOrModifiedFrom('#study-modified-from', study, 'study', MODIFIED_FROM);
+            SetupAttachments('#study-access', 'study-attachment', pageData.uuid, 'studies');
+
+            target = $('#list-series');
+            $('li', target).remove();
+            for (var i = 0; i < series.length; i++) {
+              if (i == 0 ||
+                  GetMainDicomTag(series[i].MainDicomTags, SERIES_DATE) !=
+                  GetMainDicomTag(series[i - 1].MainDicomTags, SERIES_DATE))
+              {
+                target.append($('<li>')
+                              .attr('data-role', 'list-divider')
+                              .text(FormatDicomDate(GetMainDicomTag(series[i].MainDicomTags, SERIES_DATE))));
+              }
+              
+              target.append(FormatSeries(series[i], '#series?uuid=' + series[i].ID));
+            }
+            target.listview('refresh');
+
+
+            currentPage = 'study';
+            currentUuid = pageData.uuid;
+          });
         });
       });
     });
@@ -761,37 +933,41 @@ function RefreshSeries()
   if ($.mobile.pageData) {
     pageData = DeepCopy($.mobile.pageData);
 
-    GetResource('/series/' + pageData.uuid, function(series) {
-      GetResource('/studies/' + series.ParentStudy, function(study) {
-        GetResource('/patients/' + study.ParentPatient, function(patient) {
-          GetResource('/series/' + pageData.uuid + '/instances', function(instances) {
-            Sort(instances, function(x) { return x.IndexInSeries; }, true, false);
+    GetResource('/system', function(system) {
+      GetResource('/series/' + pageData.uuid + '?full', function(series) {
+        GetResource('/studies/' + series.ParentStudy + '?full', function(study) {
+          GetResource('/patients/' + study.ParentPatient + '?full', function(patient) {
+            GetResource('/series/' + pageData.uuid + '/instances?full', function(instances) {
+              Sort(instances, function(x) { return x.IndexInSeries; }, true, false);
 
-            $('#series .patient-link').attr('href', '#patient?uuid=' + patient.ID);
-            $('#series .study-link').attr('href', '#study?uuid=' + study.ID);
+              $('#series .patient-link').attr('href', '#patient?uuid=' + patient.ID);
+              $('#series .study-link').attr('href', '#study?uuid=' + study.ID);
 
-            $('#series-info li').remove();
-            $('#series-info')
-              .append('<li data-role="list-divider">Patient</li>')
-              .append(FormatPatient(patient, '#patient?uuid=' + patient.ID, true))
-              .append('<li data-role="list-divider">Study</li>')
-              .append(FormatStudy(study, '#study?uuid=' + study.ID, true))
-              .append('<li data-role="list-divider">Series</li>')
-              .append(FormatSeries(series))
-              .listview('refresh');
+              $('#series-info li').remove();
+              var info = $('#series-info')
+                  .append('<li data-role="list-divider">Patient</li>')
+                  .append(FormatPatient(patient, '#patient?uuid=' + patient.ID, true))
+                  .append('<li data-role="list-divider">Study</li>')
+                  .append(FormatStudy(study, '#study?uuid=' + study.ID, true))
+                  .append('<li data-role="list-divider">Series</li>')
+                  .append(FormatSeries(series));
+              ConfigureLabels(info, system, 'series', series.ID);
+              info.listview('refresh');
 
-            SetupAnonymizedOrModifiedFrom('#series-anonymized-from', series, 'series', 'AnonymizedFrom');
-            SetupAnonymizedOrModifiedFrom('#series-modified-from', series, 'series', 'ModifiedFrom');
+              SetupAnonymizedOrModifiedFrom('#series-anonymized-from', series, 'series', ANONYMIZED_FROM);
+              SetupAnonymizedOrModifiedFrom('#series-modified-from', series, 'series', MODIFIED_FROM);
+              SetupAttachments('#series-access', 'series-attachment', pageData.uuid, 'series');
 
-            target = $('#list-instances');
-            $('li', target).remove();
-            for (var i = 0; i < instances.length; i++) {
-              target.append(FormatInstance(instances[i], '#instance?uuid=' + instances[i].ID));
-            }
-            target.listview('refresh');
+              target = $('#list-instances');
+              $('li', target).remove();
+              for (var i = 0; i < instances.length; i++) {
+                target.append(FormatInstance(instances[i], '#instance?uuid=' + instances[i].ID));
+              }
+              target.listview('refresh');
 
-            currentPage = 'series';
-            currentUuid = pageData.uuid;
+              currentPage = 'series';
+              currentUuid = pageData.uuid;
+            });
           });
         });
       });
@@ -800,25 +976,6 @@ function RefreshSeries()
 }
 
 
-function EscapeHtml(value)
-{
-  var ENTITY_MAP = {
-    '&': '&amp;',
-    '<': '&lt;',
-    '>': '&gt;',
-    '"': '&quot;',
-    "'": '&#39;',
-    '/': '&#x2F;',
-    '`': '&#x60;',
-    '=': '&#x3D;'
-  };
-
-  return String(value).replace(/[&<>"'`=\/]/g, function (s) {
-    return ENTITY_MAP[s];
-  });
-}
-
-
 function ConvertForTree(dicom)
 {
   var result = [];
@@ -826,14 +983,26 @@ function ConvertForTree(dicom)
 
   for (var i in dicom) {
     if (dicom[i] != null) {
-      label = (i + '<span class="tag-name"> (<i>' +
-                   EscapeHtml(dicom[i]["Name"]) +
-                   '</i>)</span>: ');
+      var spanElement = $("<span>", {
+        class:"tag-name"
+      });
+      var iElement = $("<i>", {
+        text: dicom[i]["Name"]
+      });
+      
+      spanElement.append(" (");
+      spanElement.append(iElement);
+      spanElement.append(")");
 
+      label = (i + spanElement.prop('outerHTML') + ': ');
       if (dicom[i]["Type"] == 'String')
       {
+        var strongElement = $('<strong>', {
+          text: dicom[i]["Value"]
+        });
+
         result.push({
-          label: label + '<strong>' + EscapeHtml(dicom[i]["Value"]) + '</strong>',
+          label: label + strongElement.prop('outerHTML'),
           children: []
         });
       }
@@ -856,7 +1025,7 @@ function ConvertForTree(dicom)
         c = [];
         for (var j = 0; j < dicom[i]["Value"].length; j++) {
           c.push({
-            label: 'Item ' + j,
+            label: 'Item ' + (j + 1),
             children: ConvertForTree(dicom[i]["Value"][j])
           });
         }
@@ -880,49 +1049,54 @@ function RefreshInstance()
   if ($.mobile.pageData) {
     pageData = DeepCopy($.mobile.pageData);
 
-    GetResource('/instances/' + pageData.uuid, function(instance) {
-      GetResource('/series/' + instance.ParentSeries, function(series) {
-        GetResource('/studies/' + series.ParentStudy, function(study) {
-          GetResource('/patients/' + study.ParentPatient, function(patient) {
+    GetResource('/system', function(system) {
+      GetResource('/instances/' + pageData.uuid + '?full', function(instance) {
+        GetResource('/series/' + instance.ParentSeries + '?full', function(series) {
+          GetResource('/studies/' + series.ParentStudy + '?full', function(study) {
+            GetResource('/patients/' + study.ParentPatient + '?full', function(patient) {
 
-            $('#instance .patient-link').attr('href', '#patient?uuid=' + patient.ID);
-            $('#instance .study-link').attr('href', '#study?uuid=' + study.ID);
-            $('#instance .series-link').attr('href', '#series?uuid=' + series.ID);
-            
-            $('#instance-info li').remove();
-            $('#instance-info')
-              .append('<li data-role="list-divider">Patient</li>')
-              .append(FormatPatient(patient, '#patient?uuid=' + patient.ID, true))
-              .append('<li data-role="list-divider">Study</li>')
-              .append(FormatStudy(study, '#study?uuid=' + study.ID, true))
-              .append('<li data-role="list-divider">Series</li>')
-              .append(FormatSeries(series, '#series?uuid=' + series.ID, true))
-              .append('<li data-role="list-divider">Instance</li>')
-              .append(FormatInstance(instance))
-              .listview('refresh');
+              $('#instance .patient-link').attr('href', '#patient?uuid=' + patient.ID);
+              $('#instance .study-link').attr('href', '#study?uuid=' + study.ID);
+              $('#instance .series-link').attr('href', '#series?uuid=' + series.ID);
+              
+              $('#instance-info li').remove();
+              var info = $('#instance-info')
+                  .append('<li data-role="list-divider">Patient</li>')
+                  .append(FormatPatient(patient, '#patient?uuid=' + patient.ID, true))
+                  .append('<li data-role="list-divider">Study</li>')
+                  .append(FormatStudy(study, '#study?uuid=' + study.ID, true))
+                  .append('<li data-role="list-divider">Series</li>')
+                  .append(FormatSeries(series, '#series?uuid=' + series.ID, true))
+                  .append('<li data-role="list-divider">Instance</li>')
+                  .append(FormatInstance(instance));
+              ConfigureLabels(info, system, 'instances', instance.ID);
+              info.listview('refresh');
 
-            GetResource('/instances/' + instance.ID + '/tags', function(s) {
-              $('#dicom-tree').tree('loadData', ConvertForTree(s));
+              GetResource('/instances/' + instance.ID + '/tags', function(s) {
+                $('#dicom-tree').tree('loadData', ConvertForTree(s));
+              });
+
+              GetResource('/instances/' + instance.ID + '/header', function(s) {
+                $('#dicom-metaheader').tree('loadData', ConvertForTree(s));
+              });
+
+              $('#transfer-syntax').hide();
+              GetResource('/instances/' + instance.ID + '/metadata?expand', function(s) {
+                transferSyntax = s['TransferSyntax'];
+                if (transferSyntax !== undefined) {
+                  $('#transfer-syntax').show();
+                  $('#transfer-syntax-text').text(transferSyntax);
+                }
+              });
+
+              SetupAnonymizedOrModifiedFrom('#instance-anonymized-from', instance, 'instance', ANONYMIZED_FROM);
+              SetupAnonymizedOrModifiedFrom('#instance-modified-from', instance, 'instance', MODIFIED_FROM);
+
+              SetupAttachments('#instance-access', 'instance-attachment', pageData.uuid, 'instances');
+
+              currentPage = 'instance';
+              currentUuid = pageData.uuid;
             });
-
-            GetResource('/instances/' + instance.ID + '/header', function(s) {
-              $('#dicom-metaheader').tree('loadData', ConvertForTree(s));
-            });
-
-            $('#transfer-syntax').hide();
-            GetResource('/instances/' + instance.ID + '/metadata?expand', function(s) {
-              transferSyntax = s['TransferSyntax'];
-              if (transferSyntax !== undefined) {
-                $('#transfer-syntax').show();
-                $('#transfer-syntax-text').text(transferSyntax);
-              }
-            });
-
-            SetupAnonymizedOrModifiedFrom('#instance-anonymized-from', instance, 'instance', 'AnonymizedFrom');
-            SetupAnonymizedOrModifiedFrom('#instance-modified-from', instance, 'instance', 'ModifiedFrom');
-
-            currentPage = 'instance';
-            currentUuid = pageData.uuid;
           });
         });
       });
@@ -1057,7 +1231,7 @@ $('#instance-preview').live('click', function(e) {
           if (frames.length == 1)
           {
             // Viewing a single-frame image
-            jQuery.slimbox('../instances/' + pageData.uuid + '/preview', '', {
+            jQuery.slimbox('../instances/' + pageData.uuid + '/preview?returnUnsupportedImage', '', {
               overlayFadeDuration : 1,
               resizeDuration : 1,
               imageFadeDuration : 1
@@ -1069,7 +1243,7 @@ $('#instance-preview').live('click', function(e) {
 
             images = [];
             for (var i = 0; i < frames.length; i++) {
-              images.push([ '../instances/' + pageData.uuid + '/frames/' + i + '/preview' ]);
+              images.push([ '../instances/' + pageData.uuid + '/frames/' + i + '/preview?returnUnsupportedImage' ]);
             }
 
             jQuery.slimbox(images, 0, {
@@ -1099,7 +1273,7 @@ $('#series-preview').live('click', function(e) {
 
         images = [];
         for (var i = 0; i < instances.length; i++) {
-          images.push([ '../instances/' + instances[i].ID + '/preview',
+          images.push([ '../instances/' + instances[i].ID + '/preview?returnUnsupportedImage',
                         (i + 1).toString() + '/' + instances.length.toString() ])
         }
 
@@ -1142,13 +1316,22 @@ function ChooseDicomModality(callback)
 
         for (var i = 0; i < modalities.length; i++) {
           name = modalities[i];
-          item = $('<li>')
-            .html('<a href="#" rel="close">' + name + '</a>')
-            .attr('name', name)
+
+          var liElement = $('<li>', {
+            name: name
+          })
             .click(function() { 
               clickedModality = $(this).attr('name');
             });
-          items.append(item);
+
+          var aElement = $('<a>', {
+            href: '#',
+            rel: 'close',
+            text: name
+          })
+          liElement.append(aElement);
+
+          items.append(liElement);
         }
       }
 
@@ -1168,13 +1351,22 @@ function ChooseDicomModality(callback)
 
             for (var i = 0; i < peers.length; i++) {
               name = peers[i];
-              item = $('<li>')
-                .html('<a href="#" rel="close">' + name + '</a>')
-                .attr('name', name)
+
+              var liElement = $('<li>', {
+                name: name
+              })
                 .click(function() { 
                   clickedPeer = $(this).attr('name');
                 });
-              items.append(item);
+    
+              var aElement = $('<a>', {
+                href: '#',
+                rel: 'close',
+                text: name
+              });
+              liElement.append(aElement);
+    
+              items.append(liElement);
             }
           }
 
@@ -1223,19 +1415,23 @@ $('#instance-store,#series-store,#study-store,#patient-store').live('click', fun
     }
 
     if (url != '') {
+      /**
+       * In Orthanc <= 1.9.5, synchronous job was used, which caused a
+       * non-intuitive behavior because of AJAX timeouts on large
+       * studies. We now use an asynchronous call.
+       * https://groups.google.com/g/orthanc-users/c/r2LoAp72AWI/m/cVaFXopUBAAJ
+       **/
       $.ajax({
         url: url,
         type: 'POST',
-        dataType: 'text',
-        data: pageData.uuid,
-        async: true,  // Necessary to block UI
-        beforeSend: function() {
-          $.blockUI({ message: $(loading) });
-        },
-        complete: function(s) {
-          $.unblockUI();
-        },
-        success: function(s) {
+        data: JSON.stringify({
+          'Synchronous' : false,
+          'Resources' : [ pageData.uuid ]
+        }),
+        dataType: 'json',
+        async: false,
+        success: function(job) {
+          window.location.assign('explorer.html#job?uuid=' + job.ID);
         },
         error: function() {
           alert('Error during store');
@@ -1287,6 +1483,54 @@ $('#series-media').live('click', function(e) {
 });
 
 
+$('#patient-archive-link').live('click', function(e) {
+  e.preventDefault();
+  var url = new URL('../patients/' + $.mobile.pageData.uuid + '/archive', window.location.href);
+  navigator.clipboard.writeText(url.href);
+  $(e.target).closest('li').buttonMarkup({ icon: 'check' });
+});
+
+$('#study-archive-link').live('click', function(e) {
+  e.preventDefault();
+  var url = new URL('../studies/' + $.mobile.pageData.uuid + '/archive', window.location.href);
+  navigator.clipboard.writeText(url.href);
+  $(e.target).closest('li').buttonMarkup({ icon: 'check' });
+});
+
+$('#series-archive-link').live('click', function(e) {
+  e.preventDefault();
+  var url = new URL('../series/' + $.mobile.pageData.uuid + '/archive', window.location.href);
+  navigator.clipboard.writeText(url.href);
+  $(e.target).closest('li').buttonMarkup({ icon: 'check' });
+});
+
+$('#instance-download-link').live('click', function(e) {
+  e.preventDefault();
+  var url = new URL('../instances/' + $.mobile.pageData.uuid + '/file', window.location.href);
+  navigator.clipboard.writeText(url.href);
+  $(e.target).closest('li').buttonMarkup({ icon: 'check' });
+});
+
+
+$('.patient-attachment').live('click', function(e) {
+  e.preventDefault();  //stop the browser from following
+  window.location.href = '../patients/' + $.mobile.pageData.uuid + '/attachments/' + e.target.id + '/data';
+});
+
+$('.study-attachment').live('click', function(e) {
+  e.preventDefault();  //stop the browser from following
+  window.location.href = '../studies/' + $.mobile.pageData.uuid + '/attachments/' + e.target.id + '/data';
+});
+
+$('.series-attachment').live('click', function(e) {
+  e.preventDefault();  //stop the browser from following
+  window.location.href = '../series/' + $.mobile.pageData.uuid + '/attachments/' + e.target.id + '/data';
+});
+
+$('.instance-attachment').live('click', function(e) {
+  e.preventDefault();  //stop the browser from following
+  window.location.href = '../instances/' + $.mobile.pageData.uuid + '/attachments/' + e.target.id + '/data';
+});
 
 $('#protection').live('change', function(e) {
   var isProtected = e.target.value == "on";
