@@ -2,8 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2022 Osimis S.A., Belgium
- * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -28,6 +28,7 @@
 
 #include "../../../OrthancFramework/Sources/DicomFormat/DicomArray.h"
 #include "../../../OrthancFramework/Sources/OrthancException.h"
+#include "../../../OrthancFramework/Sources/DicomParsing/FromDcmtkBridge.h"
 
 #include <cassert>
 
@@ -45,8 +46,7 @@ namespace Orthanc
       const DicomElement& element = flattened.GetElement(i);
       const DicomTag& tag = element.GetTag();
       const DicomValue& value = element.GetValue();
-      if (!value.IsNull() && 
-          !value.IsBinary())
+      if (value.IsString())
       {
         target.AddMainDicomTag(resource, tag, element.GetValue().GetContent());
       }
@@ -70,9 +70,7 @@ namespace Orthanc
       assert(DicomMap::IsMainDicomTag(tags[i]));
         
       const DicomValue* value = map.TestAndGetValue(tags[i]);
-      if (value != NULL &&
-          !value->IsNull() &&
-          !value->IsBinary())
+      if (value != NULL && value->IsString())
       {
         std::string s = ServerToolbox::NormalizeIdentifier(value->GetContent());
         target.AddIdentifierTag(resource, tags[i], s);
@@ -133,7 +131,7 @@ namespace Orthanc
         throw OrthancException(ErrorCode_InternalError);
     }
 
-    StoreMainDicomTagsInternal(*this, resource, tags);
+    StoreMainDicomTagsInternal(*this, resource, tags);  // saves only leaf tags, not sequences
   }
 
 
@@ -142,13 +140,13 @@ namespace Orthanc
     for (std::list<TagValue>::const_iterator
            it = tags_.begin(); it != tags_.end(); ++it)
     {
-      if (it->isIdentifier_)
+      if (it->IsIdentifier())
       {
-        compatibility.SetIdentifierTag(it->resourceId_, it->tag_,  it->value_);
+        compatibility.SetIdentifierTag(it->GetResourceId(), it->GetTag(),  it->GetValue());
       }
       else
       {
-        compatibility.SetMainDicomTag(it->resourceId_, it->tag_,  it->value_);
+        compatibility.SetMainDicomTag(it->GetResourceId(), it->GetTag(),  it->GetValue());
       }
     }
 
@@ -156,7 +154,7 @@ namespace Orthanc
            it = metadata_.begin(); it != metadata_.end(); ++it)
     {
       assert(isNewResource_);
-      compatibility.SetMetadata(it->resourceId_, it->metadata_,  it->value_, 0 /* initial revision number */);
+      compatibility.SetMetadata(it->GetResourceId(), it->GetType(),  it->GetValue(), 0 /* initial revision number */);
     }
   }
 }
