@@ -2,8 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2022 Osimis S.A., Belgium
- * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -186,9 +186,9 @@ namespace Orthanc
       OrthancConfiguration::DefaultExtractDicomSummary(summary, dataset_);
     }
 
-    virtual void GetDicomAsJson(Json::Value& dicomAsJson) const ORTHANC_OVERRIDE
+    virtual void GetDicomAsJson(Json::Value& dicomAsJson, const std::set<DicomTag>& ignoreTagLength) const ORTHANC_OVERRIDE
     {
-      OrthancConfiguration::DefaultDicomDatasetToJson(dicomAsJson, dataset_);
+      OrthancConfiguration::DefaultDicomDatasetToJson(dicomAsJson, dataset_, ignoreTagLength);
     }
 
     virtual void DatasetToJson(Json::Value& target, 
@@ -275,9 +275,9 @@ namespace Orthanc
   }
 
   
-  void DicomInstanceToStore::GetDicomAsJson(Json::Value& dicomAsJson) const
+  void DicomInstanceToStore::GetDicomAsJson(Json::Value& dicomAsJson, const std::set<DicomTag>& ignoreTagLength) const
   {
-    OrthancConfiguration::DefaultDicomDatasetToJson(dicomAsJson, GetParsedDicomFile());
+    OrthancConfiguration::DefaultDicomDatasetToJson(dicomAsJson, GetParsedDicomFile(), ignoreTagLength);
   }
 
 
@@ -300,4 +300,14 @@ namespace Orthanc
   {
     return GetParsedDicomFile().DecodeFrame(frame);
   }
+
+  void DicomInstanceToStore::CopyMetadata(const DicomInstanceToStore::MetadataMap& metadata)
+  {
+    for (MetadataMap::const_iterator it = metadata.begin(); 
+         it != metadata.end(); ++it)
+    {
+      AddMetadata(it->first.first, it->first.second, it->second);
+    }
+  }
+
 }
