@@ -24,6 +24,7 @@
 
 #include "DicomInstanceToStore.h"
 #include "ServerIndexChange.h"
+#include "JobEvent.h"
 
 #include <json/value.h>
 
@@ -42,6 +43,8 @@ namespace Orthanc
     
     virtual void SignalChange(const ServerIndexChange& change) = 0;
 
+    virtual void SignalJobEvent(const JobEvent& event) = 0;
+
     virtual bool FilterIncomingInstance(const DicomInstanceToStore& instance,
                                         const Json::Value& simplified) = 0;
 
