@@ -139,6 +139,11 @@ namespace Orthanc
         context_.mainLua_.SignalChange(change);
       }
 
+      virtual void SignalJobEvent(const JobEvent& event) ORTHANC_OVERRIDE
+      {
+        context_.mainLua_.SignalJobEvent(event);
+      }
+
       virtual bool FilterIncomingInstance(const DicomInstanceToStore& instance,
                                           const Json::Value& simplified) ORTHANC_OVERRIDE
       {
@@ -184,6 +189,9 @@ namespace Orthanc
     static void ChangeThread(ServerContext* that,
                              unsigned int sleepDelay);
 
+    static void JobEventsThread(ServerContext* that,
+                                unsigned int sleepDelay);
+
     static void SaveJobsThread(ServerContext* that,
                                unsigned int sleepDelay);
 
@@ -233,7 +241,9 @@ namespace Orthanc
     bool haveJobsChanged_;
     bool isJobsEngineUnserialized_;
     SharedMessageQueue  pendingChanges_;
+    SharedMessageQueue  pendingJobEvents_;
     boost::thread  changeThread_;
+    boost::thread  jobEventsThread_;
     boost::thread  saveJobsThread_;
     boost::thread  memoryTrimmingThread_;
         
@@ -272,8 +282,6 @@ namespace Orthanc
                                       StoreInstanceMode mode,
                                       bool isReconstruct);
 
-    void PublishDicomCacheMetrics();
-
     // This method must only be called from "ServerIndex"!
     void RemoveFile(const std::string& fileUuid,
                     FileContentType type);
@@ -369,6 +377,10 @@ namespace Orthanc
     void ReadDicom(std::string& dicom,
                    const std::string& instancePublicId);
 
+    void ReadDicom(std::string& dicom,
+                   std::string& attachmentId,
+                   const std::string& instancePublicId);
+
     void ReadDicomForHeader(std::string& dicom,
                             const std::string& instancePublicId);
 
@@ -378,6 +390,7 @@ namespace Orthanc
     // This method is for low-level operations on "/instances/.../attachments/..."
     void ReadAttachment(std::string& result,
                         int64_t& revision,
+                        std::string& attachmentId,
                         const std::string& instancePublicId,
                         FileContentType content,
                         bool uncompressIfNeeded,
@@ -546,6 +559,12 @@ namespace Orthanc
                            const std::set<DicomTransferSyntax>& allowedSyntaxes,
                            bool allowNewSopInstanceUid) ORTHANC_OVERRIDE;
 
+    virtual bool TranscodeWithCache(std::string& target,
+                                    const std::string& source,
+                                    const std::string& sourceInstanceId,
+                                    const std::string& attachmentId, // for the storage cache
+                                    DicomTransferSyntax targetSyntax);
+
     bool IsTranscodeDicomProtocol() const
     {
       return transcodeDicomProtocol_;
@@ -594,5 +613,7 @@ namespace Orthanc
     }
 
     int64_t GetServerUpTime() const;
+
+    void PublishCacheMetrics();
   };
 }
