@@ -2,8 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2022 Osimis S.A., Belgium
- * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -49,7 +49,14 @@ namespace Orthanc
     StoreStatus_Success,
     StoreStatus_AlreadyStored,
     StoreStatus_Failure,
-    StoreStatus_FilteredOut     // Removed by NewInstanceFilter or ReceivedInstanceCallback
+    StoreStatus_FilteredOut,     // Removed by NewInstanceFilter or ReceivedInstanceCallback
+    StoreStatus_StorageFull      // new in Orthanc 1.11.2
+  };
+
+  enum MaxStorageMode
+  {
+    MaxStorageMode_Recycle,
+    MaxStorageMode_Reject
   };
 
   enum DicomTagType
@@ -151,7 +158,10 @@ namespace Orthanc
     MetadataType_Instance_CalledAet = 12,        // New in Orthanc 1.4.0
     MetadataType_Instance_HttpUsername = 13,     // New in Orthanc 1.4.0
     MetadataType_Instance_PixelDataOffset = 14,  // New in Orthanc 1.9.0
-
+    MetadataType_MainDicomTagsSignature = 15,    // New in Orthanc 1.11.0
+    MetadataType_MainDicomSequences = 16,        // New in Orthanc 1.11.1
+    MetadataType_Instance_PixelDataVR = 17,      // New in Orthanc 1.12.1
+    
     // Make sure that the value "65535" can be stored into this enumeration
     MetadataType_StartUser = 1024,
     MetadataType_EndUser = 65535
@@ -190,6 +200,12 @@ namespace Orthanc
     BuiltinDecoderTranscoderOrder_Disabled
   };
 
+  enum Warnings
+  {
+    Warnings_None,
+    Warnings_001_TagsBeingReadFromStorage,
+    Warnings_002_InconsistentDicomTagsInDb,
+  };
 
 
   void InitializeServerEnumerations();
@@ -201,6 +217,8 @@ namespace Orthanc
 
   std::string EnumerationToString(MetadataType type);
 
+  void GetRegisteredUserMetadata(std::map<std::string, int>& allEntries);
+
   void RegisterUserContentType(int contentType,
                                const std::string& name,
                                const std::string& mime);
@@ -209,6 +227,12 @@ namespace Orthanc
 
   FindStorageAccessMode StringToFindStorageAccessMode(const std::string& str);
 
+  MaxStorageMode StringToMaxStorageMode(const std::string& str);
+  
+  bool IsStorageAccessAllowedForAnswers(FindStorageAccessMode mode);
+
+  bool IsStorageAccessAllowedForLookup(FindStorageAccessMode mode);
+
   BuiltinDecoderTranscoderOrder StringToBuiltinDecoderTranscoderOrder(const std::string& str);
 
   Verbosity StringToVerbosity(const std::string& str);
