@@ -2,8 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2022 Osimis S.A., Belgium
- * Copyright (C) 2021-2022 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -45,8 +45,9 @@ namespace Orthanc
 
     LeastRecentlyUsedIndex<int64_t, UnstableResourcePayload>  unstableResources_;
 
-    uint64_t     maximumStorageSize_;
-    unsigned int maximumPatients_;
+    MaxStorageMode  maximumStorageMode_;
+    uint64_t        maximumStorageSize_;
+    unsigned int    maximumPatients_;
 
     static void FlushThread(ServerIndex* that,
                             unsigned int threadSleep);
@@ -75,6 +76,8 @@ namespace Orthanc
     // "count == 0" means no limit on the number of patients
     void SetMaximumPatientCount(unsigned int count);
 
+    void SetMaximumStorageMode(MaxStorageMode mode);
+
     StoreStatus Store(std::map<MetadataType, std::string>& instanceMetadata,
                       const DicomMap& dicomSummary,
                       const Attachments& attachments,
@@ -84,7 +87,9 @@ namespace Orthanc
                       bool hasTransferSyntax,
                       DicomTransferSyntax transferSyntax,
                       bool hasPixelDataOffset,
-                      uint64_t pixelDataOffset);
+                      uint64_t pixelDataOffset,
+                      ValueRepresentation pixelDataVR,
+                      bool isReconstruct);
 
     StoreStatus AddAttachment(int64_t& newRevision /*out*/,
                               const FileInfo& attachment,
